import { Component, ChangeDetectorRef, SimpleChange, ComponentFactoryResolver, Injectable, Directive, EventEmitter, Input, Output, ViewContainerRef, Pipe, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const PRIVATE_PREFIX = '__ngxOnChanges_';
/**
 * @param {?} delegateHook
 * @return {?}
 */
function onChangesWrapper(delegateHook) {
    return function () {
        const /** @type {?} */ simpleChanges = this[PRIVATE_PREFIX];
        if (simpleChanges != null) {
            if (this.ngOnChanges) {
                this.ngOnChanges(simpleChanges);
            }
            this[PRIVATE_PREFIX] = null;
        }
        if (delegateHook) {
            delegateHook.apply(this);
        }
    };
}
/**
 * @param {?} delegateHook
 * @param {?} cd
 * @return {?}
 */
function markForCheckWrapper(delegateHook, cd) {
    return function () {
        if (delegateHook) {
            delegateHook.apply(this);
        }
        if (cd) {
            cd.markForCheck();
        }
    };
}
/**
 * @param {?} context
 * @param {?} name
 * @return {?}
 */
function hasProperty(context, name) {
    if (context.hasOwnProperty(name)) {
        return true;
    }
    const /** @type {?} */ prototype = Object.getPrototypeOf(context);
    if (prototype) {
        return hasProperty(prototype, name);
    }
    return false;
}
/**
 * @param {?} context
 * @param {?} name
 * @return {?}
 */
function getPropertyDescriptor(context, name) {
    if (context.hasOwnProperty(name)) {
        return Object.getOwnPropertyDescriptor(context, name);
    }
    const /** @type {?} */ prototype = Object.getPrototypeOf(context);
    if (prototype) {
        return getPropertyDescriptor(prototype, name);
    }
    return void 0;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class OnInitOnlyComponent {
    /**
     * @return {?}
     */
    ngOnInit() { }
}
OnInitOnlyComponent.decorators = [
    { type: Component, args: [{ selector: 'ngx-component-outlet-on-init-only', template: '' },] },
];
class DoCheckOnlyComponent {
    /**
     * @return {?}
     */
    ngDoCheck() { }
}
DoCheckOnlyComponent.decorators = [
    { type: Component, args: [{ selector: 'ngx-component-outlet-do-check-only', template: '' },] },
];
class OnInitAndDoCheckComponent {
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @return {?}
     */
    ngDoCheck() { }
}
OnInitAndDoCheckComponent.decorators = [
    { type: Component, args: [{ selector: 'ngx-component-outlet-on-init-do-check', template: '' },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 * @template TComponent
 */

/**
 * @template TComponent
 */
class NgxComponentOutletAdapterRef {
    /**
     * @param {?} config
     */
    constructor(config) {
        this.attachedOutputs = [];
        this.defaultDescriptors = [];
        this.componentFactory = config.componentFactory;
        this.componentRef = config.componentRef;
        this.context = config.context;
        this.onInitComponentRef = config.onInitComponentRef || /** @type {?} */ (this.componentRef);
        this.doCheckComponentRef = config.doCheckComponentRef || /** @type {?} */ (this.componentRef);
        this.changeDetectorRef = this.componentRef.injector.get(ChangeDetectorRef, this.componentRef.changeDetectorRef);
        this.attachInputs();
        this.attachLifecycle();
        this.attachOutputs();
    }
    /**
     * @return {?}
     */
    dispose() {
        this.disposeOutputs();
        this.disposeInputs();
        if (this.componentRef) {
            this.componentRef.destroy();
            this.componentRef = null;
        }
        if (this.onInitComponentRef) {
            this.onInitComponentRef.destroy();
            this.onInitComponentRef = null;
        }
        if (this.doCheckComponentRef) {
            this.doCheckComponentRef.destroy();
            this.doCheckComponentRef = null;
        }
    }
    /**
     * @return {?}
     */
    attachInputs() {
        const /** @type {?} */ inputs = this.componentFactory.inputs;
        this.defaultDescriptors = inputs.map((property) => {
            const /** @type {?} */ defaultDescriptor = getPropertyDescriptor(this.context, property.templateName);
            this.attachInput(this.context, this.componentRef.instance, property, defaultDescriptor);
            return /** @type {?} */ (Object.assign({}, property, { defaultDescriptor }));
        });
    }
    /**
     * @param {?} context
     * @param {?} instance
     * @param {?} __2
     * @param {?} defaultDescriptor
     * @return {?}
     */
    attachInput(context, instance, { propName, templateName }, defaultDescriptor) {
        const /** @type {?} */ defaultValue = context[templateName];
        Object.defineProperty(context, templateName, {
            get: () => {
                if (defaultDescriptor && defaultDescriptor.get) {
                    return defaultDescriptor.get.call(context);
                }
                else {
                    return instance[propName];
                }
            },
            set: (value) => {
                let /** @type {?} */ simpleChanges = instance[PRIVATE_PREFIX];
                if (simpleChanges == null) {
                    simpleChanges = instance[PRIVATE_PREFIX] = {};
                }
                const /** @type {?} */ isFirstChange = !(instance[`${PRIVATE_PREFIX}_${templateName}`]);
                instance[`${PRIVATE_PREFIX}_${templateName}`] = true;
                simpleChanges[templateName] = new SimpleChange(instance[propName], value, isFirstChange);
                if (defaultDescriptor && defaultDescriptor.set) {
                    defaultDescriptor.set.call(context, value);
                }
                instance[propName] = value;
            }
        });
        if (typeof defaultValue !== 'undefined') {
            context[templateName] = defaultValue;
        }
    }
    /**
     * @return {?}
     */
    attachLifecycle() {
        const /** @type {?} */ instance = /** @type {?} */ (this.componentRef.instance);
        if (hasProperty(this.componentRef.componentType.prototype, 'ngOnChanges')) {
            const /** @type {?} */ markForCheckWrapped = markForCheckWrapper(instance.ngDoCheck, this.changeDetectorRef).bind(instance);
            this.onInitComponentRef.instance.ngOnInit = onChangesWrapper(instance.ngOnInit).bind(instance);
            this.doCheckComponentRef.instance.ngDoCheck = onChangesWrapper(markForCheckWrapped).bind(instance);
        }
        else {
            this.doCheckComponentRef.instance.ngDoCheck = markForCheckWrapper(instance.ngDoCheck, this.changeDetectorRef).bind(instance);
        }
    }
    /**
     * @return {?}
     */
    disposeInputs() {
        const /** @type {?} */ instance = /** @type {?} */ (this.componentRef.instance);
        this.defaultDescriptors.forEach(({ propName, templateName, defaultDescriptor }) => {
            if (defaultDescriptor) {
                if (defaultDescriptor.writable) {
                    defaultDescriptor.value = instance[propName];
                }
                Object.defineProperty(this.context, templateName, defaultDescriptor);
                if (defaultDescriptor.set) {
                    this.context[templateName] = instance[propName];
                }
            }
            else {
                delete this.context[templateName];
                this.context[templateName] = instance[propName];
            }
        });
    }
    /**
     * @return {?}
     */
    attachOutputs() {
        const /** @type {?} */ availableOutputs = this.componentFactory.outputs.filter((property) => this.context.hasOwnProperty(property.templateName));
        this.attachedOutputs = availableOutputs.map((property) => this.componentRef.instance[property.propName].subscribe(this.context[property.templateName]));
    }
    /**
     * @return {?}
     */
    disposeOutputs() {
        this.attachedOutputs.splice(0).forEach((subscription) => subscription.unsubscribe());
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/** @enum {number} */
const LifecycleStrategyType = {
    Default: 0,
    OnInitOnly: 1,
    DoCheckOnly: 2,
    OnInitAndDoCheck: 3,
};
LifecycleStrategyType[LifecycleStrategyType.Default] = "Default";
LifecycleStrategyType[LifecycleStrategyType.OnInitOnly] = "OnInitOnly";
LifecycleStrategyType[LifecycleStrategyType.DoCheckOnly] = "DoCheckOnly";
LifecycleStrategyType[LifecycleStrategyType.OnInitAndDoCheck] = "OnInitAndDoCheck";
/**
 * @record
 */

const STRATEGY_CONFIG = {
    [LifecycleStrategyType.Default]: {},
    [LifecycleStrategyType.OnInitOnly]: {
        componentType: OnInitOnlyComponent,
        useOnInitComponent: true
    },
    [LifecycleStrategyType.DoCheckOnly]: {
        componentType: DoCheckOnlyComponent,
        useDoCheckComponent: true
    },
    [LifecycleStrategyType.OnInitAndDoCheck]: {
        componentType: OnInitAndDoCheckComponent,
        useOnInitComponent: true,
        useDoCheckComponent: true
    }
};
class NgxComponentOutletAdapterBuilderStrategy {
    /**
     * @param {?} componentFactoryResolver
     * @param {?} config
     */
    constructor(componentFactoryResolver, config) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.config = config;
    }
    /**
     * @template TComponent
     * @param {?} componentFactory
     * @param {?} componentRef
     * @param {?} viewContainerRef
     * @param {?} context
     * @return {?}
     */
    create(componentFactory, componentRef, viewContainerRef, context) {
        const { onInitComponentRef, doCheckComponentRef } = this.createFeatureComponents(viewContainerRef);
        return this.createAdapterRef({
            componentFactory, componentRef, context,
            onInitComponentRef, doCheckComponentRef
        });
    }
    /**
     * @param {?} viewContainerRef
     * @return {?}
     */
    createFeatureComponents(viewContainerRef) {
        if (!this.config.componentType) {
            return { onInitComponentRef: null, doCheckComponentRef: null };
        }
        const /** @type {?} */ featureComponentFactory = this.componentFactoryResolver.resolveComponentFactory(this.config.componentType);
        const /** @type {?} */ featureComponentRef = viewContainerRef.createComponent(featureComponentFactory, viewContainerRef.length);
        return {
            onInitComponentRef: this.config.useOnInitComponent ? featureComponentRef : null,
            doCheckComponentRef: this.config.useDoCheckComponent ? featureComponentRef : null
        };
    }
    /**
     * @template TComponent
     * @param {?} config
     * @return {?}
     */
    createAdapterRef(config) {
        return new NgxComponentOutletAdapterRef(config);
    }
}
class NgxComponentOutletAdapterBuilderStrategyResolver {
    /**
     * @param {?} componentFactoryResolver
     */
    constructor(componentFactoryResolver) {
        this.componentFactoryResolver = componentFactoryResolver;
    }
    /**
     * @param {?} component
     * @return {?}
     */
    resolve(component) {
        const /** @type {?} */ type = this.getStrategyType(component);
        return new NgxComponentOutletAdapterBuilderStrategy(this.componentFactoryResolver, STRATEGY_CONFIG[type]);
    }
    /**
     * @param {?} component
     * @return {?}
     */
    getStrategyType(component) {
        const /** @type {?} */ hasNgOnInit = hasProperty(component.prototype, 'ngOnInit');
        const /** @type {?} */ hasNgDoCheck = hasProperty(component.prototype, 'ngDoCheck');
        const /** @type {?} */ hasNgOnChanges = hasProperty(component.prototype, 'ngOnChanges');
        if (hasNgOnChanges && !hasNgOnInit && !hasNgDoCheck) {
            return LifecycleStrategyType.OnInitAndDoCheck;
        }
        if (hasNgOnChanges && !hasNgOnInit) {
            return LifecycleStrategyType.OnInitOnly;
        }
        if (!hasNgDoCheck) {
            return LifecycleStrategyType.DoCheckOnly;
        }
        return LifecycleStrategyType.Default;
    }
}
NgxComponentOutletAdapterBuilderStrategyResolver.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgxComponentOutletAdapterBuilderStrategyResolver.ctorParameters = () => [
    { type: ComponentFactoryResolver, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgxComponentOutletAdapterBuilder {
    /**
     * @param {?} adapterBuilderStrategyResolver
     */
    constructor(adapterBuilderStrategyResolver) {
        this.adapterBuilderStrategyResolver = adapterBuilderStrategyResolver;
    }
    /**
     * @template TComponent
     * @param {?} componentType
     * @param {?} viewContainerRef
     * @param {?} injector
     * @param {?} projectableNodes
     * @param {?} context
     * @param {?} componentFactoryResolver
     * @return {?}
     */
    create(componentType, viewContainerRef, injector, projectableNodes, context, componentFactoryResolver) {
        const /** @type {?} */ componentFactory = componentFactoryResolver.resolveComponentFactory(componentType);
        const /** @type {?} */ componentRef = viewContainerRef.createComponent(componentFactory, viewContainerRef.length, injector, projectableNodes);
        const /** @type {?} */ strategy = this.adapterBuilderStrategyResolver.resolve(componentType);
        return strategy.create(componentFactory, componentRef, viewContainerRef, context);
    }
}
NgxComponentOutletAdapterBuilder.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgxComponentOutletAdapterBuilder.ctorParameters = () => [
    { type: NgxComponentOutletAdapterBuilderStrategyResolver, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgxComponentOutlet {
    /**
     * @param {?} _componentFactoryResolver
     * @param {?} viewContainerRef
     * @param {?} builder
     */
    constructor(_componentFactoryResolver, viewContainerRef, builder) {
        this._componentFactoryResolver = _componentFactoryResolver;
        this.viewContainerRef = viewContainerRef;
        this.builder = builder;
        this.ngxComponentOutletActivate = new EventEmitter();
        this.ngxComponentOutletDeactivate = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get componentFactoryResolver() {
        return this._ngModuleRef ? this._ngModuleRef.componentFactoryResolver : this._componentFactoryResolver;
    }
    /**
     * @return {?}
     */
    get context() {
        const { context } = /** @type {?} */ (this.injector.get(ChangeDetectorRef));
        return context;
    }
    /**
     * @return {?}
     */
    get injector() {
        return this.ngxComponentOutletInjector || this.viewContainerRef.injector;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes["ngxComponentOutlet"]) {
            if (changes["ngxComponentOutletNgModuleFactory"]) {
                this.destroyNgModuleRef();
                this.createNgModuleRef();
            }
            this.destroyAdapterRef();
            this.createAdapterRef();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyAdapterRef();
        this.destroyNgModuleRef();
    }
    /**
     * @return {?}
     */
    createAdapterRef() {
        if (this.ngxComponentOutlet) {
            this._adapterRef = this.builder.create(this.ngxComponentOutlet, this.viewContainerRef, this.injector, this.ngxComponentOutletContent, this.context, this.componentFactoryResolver);
            this.ngxComponentOutletActivate.emit(this._adapterRef.componentRef.instance);
        }
    }
    /**
     * @return {?}
     */
    destroyAdapterRef() {
        if (this._adapterRef) {
            this.ngxComponentOutletDeactivate.emit(this._adapterRef.componentRef.instance);
            this._adapterRef.dispose();
            this._adapterRef = null;
        }
    }
    /**
     * @return {?}
     */
    createNgModuleRef() {
        if (this.ngxComponentOutletNgModuleFactory) {
            this._ngModuleRef = this.ngxComponentOutletNgModuleFactory.create(this.injector);
        }
    }
    /**
     * @return {?}
     */
    destroyNgModuleRef() {
        if (this._ngModuleRef) {
            this._ngModuleRef.destroy();
            this._ngModuleRef = null;
        }
    }
}
NgxComponentOutlet.decorators = [
    { type: Directive, args: [{ selector: '[ngxComponentOutlet]' },] },
];
/** @nocollapse */
NgxComponentOutlet.ctorParameters = () => [
    { type: ComponentFactoryResolver, },
    { type: ViewContainerRef, },
    { type: NgxComponentOutletAdapterBuilder, },
];
NgxComponentOutlet.propDecorators = {
    "ngxComponentOutlet": [{ type: Input },],
    "ngxComponentOutletInjector": [{ type: Input },],
    "ngxComponentOutletContent": [{ type: Input },],
    "ngxComponentOutletNgModuleFactory": [{ type: Input },],
    "ngxComponentOutletActivate": [{ type: Output },],
    "ngxComponentOutletDeactivate": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 * @template TEntity, TComponent
 */

class NgxComponentOutletResolvePipe {
    /**
     * @template TEntity, TComponent
     * @param {?} value
     * @param {?} resolver
     * @return {?}
     */
    transform(value, resolver) {
        return resolver.resolve(value);
    }
}
NgxComponentOutletResolvePipe.decorators = [
    { type: Pipe, args: [{ name: 'resolve', pure: true },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgxComponentOutletModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: NgxComponentOutletModule,
            providers: [
                NgxComponentOutletAdapterBuilder,
                NgxComponentOutletAdapterBuilderStrategyResolver
            ]
        };
    }
    /**
     * @return {?}
     */
    static forChild() {
        return {
            ngModule: NgxComponentOutletModule
        };
    }
}
NgxComponentOutletModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    NgxComponentOutlet,
                    NgxComponentOutletResolvePipe,
                    OnInitOnlyComponent,
                    DoCheckOnlyComponent,
                    OnInitAndDoCheckComponent
                ],
                entryComponents: [
                    OnInitOnlyComponent,
                    DoCheckOnlyComponent,
                    OnInitAndDoCheckComponent
                ],
                exports: [
                    NgxComponentOutlet,
                    NgxComponentOutletResolvePipe
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { NgxComponentOutletModule, NgxComponentOutletAdapterBuilder as ɵb, DoCheckOnlyComponent as ɵf, OnInitAndDoCheckComponent as ɵg, OnInitOnlyComponent as ɵe, NgxComponentOutletAdapterBuilderStrategyResolver as ɵc, NgxComponentOutlet as ɵa, NgxComponentOutletResolvePipe as ɵd };
//# sourceMappingURL=ngx-component-outlet.js.map
